@@ -14,16 +14,23 @@
 # add these directories to sys.path here. If the directory is relative to the
 # documentation root, use os.path.abspath to make it absolute, like shown here.
 
+import datetime
 import os
+import time
 import sys
 sys.path.insert(0, os.path.abspath('..'))
 import mesonpy
 
+build_date = datetime.datetime.fromtimestamp(
+    int(os.environ.get('SOURCE_DATE_EPOCH', time.time())),
+    tz=datetime.timezone.utc,
+)
+
 
 # -- Project information -----------------------------------------------------
 
 project = 'meson-python'
-copyright = '2021, The meson-python developers'
+copyright = f'2021\N{EN DASH}{build_date.year} The meson-python developers'
 
 # The short X.Y version
 version = mesonpy.__version__
@@ -92,7 +99,7 @@ html_theme_options = {
 
 # Open Graph
 
-ogp_site_url = 'https://meson-python.readthedocs.io'
+ogp_site_url = 'https://mesonbuild.com/meson-python/'
 ogp_site_name = 'meson-python documentation'
 
 
