@@ -3,6 +3,7 @@
 # SPDX-License-Identifier: MIT
 
 import pathlib
+import re
 
 import packaging.version
 import pyproject_metadata
@@ -48,5 +49,12 @@ def test_missing_version(package_missing_version):
     pyproject = {'project': {
         'name': 'missing-version',
     }}
-    with pytest.raises(pyproject_metadata.ConfigurationError, match='Required "project.version" field is missing'):
+    match = '|'.join((
+        re.escape('Required "project.version" field is missing'),
+        # pyproject-metatadata 0.8.0 and later
+        re.escape('Field "project.version" missing and "version" not specified in "project.dynamic"'),
+        # pyproject-metatadata 0.9.0 and later
+        re.escape('Field "project.version" missing and \'version\' not specified in "project.dynamic"'),
+    ))
+    with pytest.raises(pyproject_metadata.ConfigurationError, match=match):
         Metadata.from_pyproject(pyproject, pathlib.Path())
